//
//  Action.m
//  Skip Checker
//
//  Created by Sven on 16.10.06 for SkipChecker.
//  Copyright 2006 earthlingsoft.
//	http://earthlingsoft.net/Skip%20Checker  earthlingsoft@earthlingsoft.net
//
//	Feel free to use this code at your risk and for your own amusement. 
//	If you find it helpful and can use it for yourself, don't forget to give credit.
//	If you spot problems with the code, please let us know.

#import "Action.h"


@implementation Action


/*
	Run the action IF
		* the time passed since last running it is at least delta AND
		* theLevel is above/below the triggerLevel (depending on the TriggerMode)

	Returns YES if the action was run and NO otherwise
*/
- (BOOL) runForLevel:(NSNumber *) theLevel withDelta:(NSNumber *) delta {
	NSTimeInterval now = [NSDate timeIntervalSinceReferenceDate];
	double current = [theLevel doubleValue];
	double trigger = [triggerLevel doubleValue];
		
	if (lastTriggered + [delta doubleValue] < now) {
		if ( (mode == triggerWhenGoingAbove && trigger < current) || (mode == triggerWhenGoingBelow && trigger > current) ) {
			[self runAction];
			[self setValue:[NSNumber numberWithDouble:now] forKey:LASTTRIGGERED];
			return YES;
		}
	}
	return NO;
}


/* 
	actually runs the action
*/
- (BOOL) runAction {
	NSDictionary *errorDict;
	
	if([action isKindOfClass:[NSAppleScript class]]) {
		return ([(NSAppleScript*)action executeAndReturnError:&errorDict] != nil);
	}
	else if ([action isKindOfClass:[NSSound class]]) {
		return [(NSSound*)action play];
	}		
	return NO;
}



#pragma mark KVO

/* observe changes to our action string, check whether the new text is a valid AppleScript or a path to an AppleScript file or a path to a Sound file and set up the action for the corresponding direction accordingly
*/
- (void) observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	//	NSLog(@"[Action observeValueForKeyPath:%@", keyPath);

	if (![keyPath isEqualToString:ACTIONTEXT]) {
		// we only want to handle changes to the text (and there shouldn't be any other changes coming in either)
		return;
	}

	NSString *newText = [change objectForKey:NSKeyValueChangeNewKey];

	NSAppleScript *as;
	NSDictionary * errorDict;
	NSFileManager *myFM = [NSFileManager defaultManager];
	
	if (![newText isKindOfClass:[NSNull class]]) {
		// there is some text, so analyse it
		// first expand any potential paths...
		NSString *myPath = [newText stringByExpandingTildeInPath];
		// ...and check whether there is something at that potential path
		if ([myFM isReadableFileAtPath:myPath]) {
			// we received a file path rather than a script and we can read that file as well
			// first hope that it's a working AppleScript	
			NSURL *myURL = [NSURL fileURLWithPath:myPath];
			as = [[[NSAppleScript alloc] initWithContentsOfURL:myURL error:&errorDict] autorelease];
			if (as && !errorDict) {
				// yup, got an AppleScript, so set it as the action
				[self setValue:as forKey:ACTION];
			}
			else {
				// we didn't get an AppleScript, so hope it's a sound file
				NSSound *mySound = [[[NSSound alloc] initWithContentsOfURL:myURL byReference:YES] autorelease];
				if (mySound) {
					// it was a sound file, so set the sound as the new action
					[self setValue:mySound forKey:ACTION];
					[self setValue:[NSColor blueColor] forKey:COLOUR];
				}
				else {
					// things didn't work out, so there's no action
					[self setValue:nil forKey:ACTION];
					[self setValue:[NSColor grayColor] forKey:COLOUR];
				}
			}
		}
		else {
			// we didn't receive a file path, so let's hope this was a script
			as = [[[NSAppleScript alloc] initWithSource:newText] autorelease];
			if ([as compileAndReturnError:&errorDict]) {
				// could compile -> update script and display
				[self setValue:as forKey:ACTION];
				[self setValue:[NSColor blackColor] forKey:COLOUR];
			}
			else {
				// things didn't work out, so there's no action
				[self setValue:nil forKey:ACTION];
				[self setValue:[NSColor grayColor] forKey:COLOUR];
			}
		}
	}
}





#pragma mark HOUSEKEEPING

-(id) init {
	[super init];
	
	// start observing changes
	[self addObserver:self forKeyPath:ACTIONTEXT options:(NSKeyValueObservingOptionNew) context:nil];
	
	return self;
}


/*
	Initialise Action object with the action data that was passed and the current time as the lastTriggered timestamp.
	If the action data are an NSAppleScript or an NSSound, the action itself will be generated, if the action data are an NSString, it will be set appropriately and KVO will cause the action to be generated from the string if possible.
*/
+ (id) actionWithAction:(id) theAction triggerLevel:(NSNumber *) theLevel mode:(NSNumber*) theMode {
	// initialise
	Action * a = [[[Action alloc] init] autorelease];

	// set values
	[a setValue:theLevel forKey:TRIGGERLEVEL];
	[a setValue:theMode forKey:MODE];
	[a setValue:[NSNumber numberWithDouble:[NSDate timeIntervalSinceReferenceDate]] forKey:LASTTRIGGERED];
	
	// determine the kind of action we received and initialise either the text or action field with it
	if ( [theAction isKindOfClass:[NSAppleScript class]] || [theAction isKindOfClass:[NSSound class]] ) {
		// we have a readymade object, set it
		[a setValue:theAction forKey:ACTION];
	}
	else if ( [theAction isKindOfClass:[NSString class]] ) {
		// just a string, so set that
		[a setValue:theAction forKey:ACTIONTEXT];
	}
	else {
		// we didn't receive an object we can interpret or perhaps even nil, just set us up with an empty string
		[a setValue:@"" forKey:ACTIONTEXT];
	}
	
	return a;
}



/*
	accessor methods for the mode variable
*/
- (NSNumber*) mode {
	return [NSNumber numberWithInt:mode];
}

- (void) setMode:(NSNumber*) newMode {
	if ( [newMode intValue] == triggerWhenGoingBelow ) {
		mode = triggerWhenGoingBelow;
	}
	else {
		mode = triggerWhenGoingAbove;
	}	
}



-(void) dealloc {
	// stop observing things
	[self removeObserver:self forKeyPath:ACTIONTEXT];
	
	[super dealloc];
}

@end






/* 
	Transformer
	giving a colour depending on the kind of action object we have
		grey for no action
		black for an AppleScript
		blue for a sound
*/

// THIS DIDN'T WORK OUT IN PRACTICE, SO IT ISN'T USED

/*
@implementation actionToColourTransformer

+ (Class)transformedValueClass;
{
    return [NSColor class];
}

+ (BOOL)allowsReverseTransformation;
{
    return NO;   
}

- (id)transformedValue:(id)value;
{
	NSColor * returnColour;
	
	if (![value isKindOfClass:[Action class]]) {
		// bail out
		[NSException raise: NSInternalInconsistencyException format: @"actionToColourTransformer can only handle Action objects, but a %@ was passed", [value class]];
		return nil;
    }
	
    // we've got an Action class, so grab its action
	id theAction = [value valueForKey:ACTION];
	
	if ([theAction isKindOfClass:[NSAppleScript class]]) {
		// a working AppleScript => active, black
		returnColour = [NSColor blackColor];
	}
	else if ([theAction isKindOfClass:[NSSound class]]) {
		// we got sound => active, blue
		returnColour = [NSColor blueColor];
	}
	else {	
		// no action available => inactive, grey 
		returnColour = [NSColor grayColor];
	}
		
    return returnColour;
}

@end
*/
